""" Fichier ramachandran_utils.py """

import re
import math
import Bio.PDB
import matplotlib.pyplot as plt


def ligne_titre(fichier):

    """ récupération de la ligne de titre """

    expr = re.compile("^TITLE", re.IGNORECASE)
    nb_lig = 0
    with open(fichier, "r", encoding="UTF-8") as pdb_fic:
        lignes = pdb_fic.readlines()
    for ligne in lignes:
        nb_lig += 1
        if expr.match(ligne):
            titre = ligne.strip().replace("TITLE", "")
    pdb_fic.close()
    return [nb_lig, titre]


def type_residu(residu, r_suivant):

    """ renvoie le type du résidu """

    if residu.resname.upper() == "GLY":
        return "Glycine"
    if residu.resname.upper() == "PRO":
        return "Proline"
    if r_suivant is not None and r_suivant.resname.upper() == "PRO":
        return "Preproline"

    return "General"


def analyse_res(structure):

    """ analyse des résidus par type """

    nb_res = {'General': 0, 'Glycine': 0, 'Proline': 0, 'Preproline': 0}
    coord = {'x_gen': [], 'y_gen': [], 'x_gly': [], 'y_gly': [],
             'x_pro': [], 'y_pro': [], 'x_pre': [], 'y_pre': []}

    ch_peptides = Bio.PDB.CaPPBuilder().build_peptides(structure)
    for polypeptide in ch_peptides:
        liste_angles = polypeptide.get_phi_psi_list()
        for num_residu, residu in enumerate(polypeptide):
            angle_phi, angle_psi = liste_angles[num_residu]
            if angle_phi and angle_psi:
                type_rama = type_residu(residu,
                      polypeptide[num_residu+1])
                nb_res[type_rama] += 1
                xvalue = math.degrees(angle_phi)
                yvalue = math.degrees(angle_psi)
                if type_rama == "General":
                    coord['x_gen'].append(xvalue)
                    coord['y_gen'].append(yvalue)
                elif type_rama == "Glycine":
                    coord['x_gly'].append(xvalue)
                    coord['y_gly'].append(yvalue)
                elif type_rama == "Proline":
                    coord['x_pro'].append(xvalue)
                    coord['y_pro'].append(yvalue)
                elif type_rama == "Pre-Pro":
                    coord['x_pre'].append(xvalue)
                    coord['y_pre'].append(yvalue)

    print(F" {nb_res['General']} résidus,", end="")
    print(F" {nb_res['Glycine']} glycines,", end="")
    print(F" {nb_res['Proline']} prolines,", end="")
    print(F" {nb_res['Preproline']} préprolines.")

    return coord


def rama_diagram(coord):

    """ graphique/diagramme de Ramachandran """

    nbgr = 0
    plt.figure()
    figure = plt.figure(figsize=(20, 20))
    plt.gcf().subplots_adjust(hspace=0.2)
    for nom_gr in ['General', 'Glycine', 'Proline', 'Preproline']:
        nbgr += 1
        if nom_gr == 'General':
            xdata = coord['x_gen']
            ydata = coord['y_gen']
            couls = "ko"
        elif nom_gr == 'Glycine':
            xdata = coord['x_gly']
            ydata = coord['y_gly']
            couls = "ro"
        elif nom_gr == 'Proline':
            xdata = coord['x_pro']
            ydata = coord['y_pro']
            couls = "b*"
        elif nom_gr == 'Preproline':
            xdata = coord['x_pre']
            ydata = coord['y_pre']
            couls = "g^"
        axes = figure.add_subplot(2, 2, nbgr)
        plt.plot(xdata, ydata, couls)
        plt.axhline(color='black')
        plt.axvline(color='black')
        major_ticks = range(-180, +210, 30)
        axes.set_xticks(major_ticks)
        axes.set_yticks(major_ticks)
        axes.set_xlabel("Angle angle_phi (degre)")
        axes.set_ylabel("Angle angle_psi (degre)")
        plt.grid()
        axes.set_title(nom_gr)

    plt.savefig('ramachandran.png', dpi=60, format='png')
    plt.close('all')
